// Tools
import React from 'react'
import ReactDom from 'react-dom'
import interact from 'interact.js'
import _ from 'lodash'

var findDOMNode = ReactDom.findDOMNode;

class DragSortableList extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            placeholder: null, // target item (being dragged over)
            dragging: null, // dragged item (being dragged),
            draggingData: null,
            items: []
        };
        this.ref = 'List' + _.uniqueId(); // generate ref
    }

    componentDidMount() {
        var draggableChildrenSelector = '#' + this.ref + '> .draggable';
        interact(draggableChildrenSelector).draggable({
            onmove: _.bind(this._dragMove, this),
            onend: _.bind(this._dragEnd, this)
        });
        this._initItems(this.props);
    }

    componentWillReceiveProps(newProps) {
        this._initItems(newProps);
    }

    _initItems(props) {
        var items = _.map(props.items, function(item, i) {
            item.rank = i;
            item.id = (item.id) ? item.id : _.uniqueId();
        });
        this.setState({
            items: props.items
        }, () => {
            // Force list items to fit their content
            _.forEach(this.state.items, (item) => {
                var node = this.refs['item-' + item.id];
                if(node) {
                    var child = node.children[0];
                    if(child) {
                        node.style.width = child.offsetWidth + 'px';
                        node.style.height = child.offsetHeight + 'px';
                    }
                }
            });
        });
    }

    render() {
        var listItems = _.clone(this.state.items);
        var draggedItem = null;

         // Add drag target
        if(this.state.placeholder) {
            // Filter out item being dragged
            listItems = _.filter(listItems, (item) => {
                if(this.state.dragging && item.id === this.state.dragging.id) {
                    draggedItem = item; // store it for display
                    //return false;
                }
                return true;
            });


            // Add drag target
            listItems.push(
                {
                    rank: this.state.placeholder.rank,
                    placeholder: draggedItem
                }
            );

            // Sort list
            listItems = _.sortBy(listItems, (item) => {
                return item.rank;
            });
        }

        var listItems = listItems.map((item) => {
            if(item.placeholder) {
                return this._displayItem(item.placeholder, 'placeholder');
            } else {
                var type = (draggedItem === item) ? 'dragged' : 'normal';
                return this._displayItem(item, type);
            }
        });

        return (
            <div>
                <div id={this.ref} className="List" ref={this.ref}>
                    {listItems}
                </div>
            </div>
        );
    }

    _displayItem(item, type) {
        var placeholder = (this.props.placeholder) ? this.props.placeholder : item.content;
        var key = 'item-' + item.id;

        if(type === 'normal') {
            var style = {
                display: 'block',
                position: 'relative'
            };
            if(this.props.type === 'horizontal') {
                style['float'] = 'left';
            }
            return (
                <div style={style} data-id={item.id} data-rank={item.rank} ref={key} key={key} className="draggable">{item.content}</div>
            );
        }

        if(type === 'dragged') {
            var style = {
                display: 'none',
                position: 'relative'
            };
            if(this.props.type === 'horizontal') {
                style['float'] = 'left';
            }
            return (
                <div ref={this.ref + 'dragged'} data-id={item.id} key={key} className="draggable dragged" style={style}>{item.content}</div>
            );
        }

        if(type === 'placeholder') {
            var style = {
                display: 'block',
                position: 'relative',
                width: this.state.draggingData.width,
                height: this.state.draggingData.height
            };
            if(this.props.type === 'horizontal') {
                style['float'] = 'left';
            }
            return (
                <div key={'placeholder'} className={'draggable placeholder'} style={style}>
                    {placeholder}
                </div>
            );
        }
    }

    _dragMove(event) {
        var target = event.target;

        // Move copy of dragged element
        // keep the dragged position in the data-x/data-y attributes
        var x = (parseFloat(target.getAttribute('data-x')) || 0) + event.dx;
        var y = (parseFloat(target.getAttribute('data-y')) || 0) + event.dy;

        // prepare future state
        var state = _.clone(this.state, true);
        var dragId =  target.getAttribute('data-id');
        state.dragging = {
            id: dragId
        };

        // update the position attributes
        target.setAttribute('data-x', x);
        target.setAttribute('data-y', y);

        var dragged = this.refs[this.ref + 'dragged'];
        if(dragged && this.state.draggingData) {
            // Translate dragged item
            dragged.style.display = 'block';
            dragged.style.position = 'absolute';
            dragged.style.top = this.state.draggingData.position.top + 'px';
            dragged.style.left = this.state.draggingData.position.left + 'px';
            dragged.style.webkitTransform = dragged.style.transform = dragged.style.msTransform = 'translate(' + x + 'px, ' + y + 'px)';
        } else {
            // Stor eoriginal position
            var position = target.getBoundingClientRect();
            state.draggingData =  {
                position: position,
                width: target.offsetWidth,
                height: target.offsetHeight
            };
        }

        // Update state
        if(!this.state.dragging || dragId !== this.state.dragging.id) {
            this.setState(state, () => {
                this._createplaceholder(event);
            });
        } else {
            this._createplaceholder(event);
        }
    }

    _dragEnd(event) {
        var items = this._moveItem();

        // Reset style
        var dragged = this.refs[this.ref + 'dragged'];
        dragged.style.display = null;
        dragged.style.position = null;
        dragged.style.top =null;
        dragged.style.left = null;
        dragged.style.webkitTransform = dragged.style.transform = dragged.style.msTransform = null;
        dragged.setAttribute('data-x', 0);
        dragged.setAttribute('data-y', 0);

        // Update rank
        this.setState({
            dragging: null,
            placeholder: null,
            items: items
        });

        if(this.props.onSort && _.isFunction(this.props.onSort)) {
            this.props.onSort(items);
        }
    }

    _moveItem() {
        var items = _.clone(this.state.items);

        // Replace drag ranks
        var dragged = _.find(items, {id: this.state.dragging.id});
        dragged.rank = this.state.placeholder.rank;

        items = _.sortBy(items, function(item) {
            return item.rank;
        });

        // Normalize items ranks
        var rank = 0;
        items = _.forEach(items, (item) => {
            item.rank = rank;
            rank++;
        });

        return items;
    }

    _createplaceholder(e) {
        var list = this.refs[this.ref];
        var mouseX = e.pageX;
        var mouseY = e.pageY;
        var children = _.filter(list.childNodes, (child) => {
            if(!(child.getAttribute('data-rank'))) {
                return false;
            }
            return true;
        });

        // Find placeholder
        var placeholder = null;

        _.forEach(children, (child) => {
            var position = child.getBoundingClientRect();
            var childX = (position.left + child.offsetWidth / 2);
            var childY = (position.top + child.offsetHeight / 2)
            var distanceX = Math.abs(mouseX - childX);
            var distanceY = Math.abs(mouseY - childY);
            var distance = (this.props.type === 'vertical') ? distanceY : distanceX;

            if(!placeholder || distance < placeholder.distance) {
                if(this.props.type === 'vertical') {
                    var position = (mouseY > childY) ? 'after' : 'before';
                } else {
                    var position = (mouseX > childX) ? 'after' : 'before';
                }
                if(position === 'before') {
                    var rank = parseInt(child.getAttribute('data-rank'), 10)  - 0.5;
                } else {
                    var rank = parseInt(child.getAttribute('data-rank'), 10) + 0.5;
                }

                placeholder = {
                    rank: rank,
                    distance: distance
                };
            }
        });

        this.setState({
            placeholder: placeholder
        });

    }

};

// Props
DragSortableList.propTypes = {
    items: React.PropTypes.array,
    type: React.PropTypes.string
};
DragSortableList.defaultProps = {
    items: [],
    type: 'vertical' //horizontal
};

export default DragSortableList;
